uses Crt,Service,kbConst;

type
	XRange = 1..40;
	YRange = 1..22;

const
	StartProbability = 5;
	NewProbability: Word = 0;
	OldColor: Byte = Red;
	CurColor = Brown;
	DelayTime: Integer = 0;
	StatusColor = LightGray;

	Generation: LongInt = 0;

type

	TCellState=record
		CurState,NextState: Boolean;
	end;

	TCellsArray = array[XRange,YRange] of TCellState;

var
	Cells: TCellsArray;
	X,Y: Integer;
	LastKey: Word;

procedure Start;
begin
	FillChar(Cells,SizeOf(Cells),0);
	for Y:=Low(YRange) to High(YRange) do
		for X:=Low(XRange) to High(XRange) do
			if Random(StartProbability)=0 then Cells[X,Y].CurState:=True;
	Generation:=0;
end;

procedure DrawCells(var Cells: TCellsArray);
var
	X,Y: Integer;
	CellsCount: Integer;
	ProbStr: string[15];
begin
	CellsCount:=0;
	TextColor(Green);
	for Y:=Low(YRange) to High(YRange) do
	begin
		GoToXY(1,Y);
		for X:=Low(XRange) to High(XRange) do
			if Cells[X,Y].NextState then
			begin
				TextColor(CurColor);
				Write('');
				Inc(CellsCount);
			end
			else
				if Cells[X,Y].CurState then
				begin
					TextColor(OldColor);
					Write('');
				end
				else Write('  ');
	end;
	GoToXY(1,24);
	TextColor(StatusColor);
	Writeln(
		' :',Generation:6,
		'     ⢮ ⮪:',CellsCount:5,
		'     ⭮ ⮪:',CellsCount/(High(XRange)*High(YRange))*100:6:2,'%');
	if NewProbability=0 then ProbStr:='  '
	else ProbStr:='1/';
	Write(
		'⭮ 砩 ஦ : ',ProbStr,NewProbability:5,
		'    প :',DelayTime:4,' ');
	ClrEol;
	Inc(Generation);
end;

function NeighboursCount(X,Y: Integer; var Cells: TCellsArray): Integer;
var
	Result,xx,yy: Integer;
begin
	Result:=0;
	for yy:=Pred(Y) to Succ(Y) do
		for xx:=Pred(X) to Succ(X) do
			if (xx>=Low(XRange)) and (yy>=Low(YRange)) and
				(xx<=High(XRange)) and (yy<=High(YRange)) and
				((xx<>X) or (yy<>Y)) then Inc(Result,Byte(Cells[xx,yy].CurState));
	NeighboursCount:=Result;
end;

procedure ResetCells(var Cells: TCellsArray);
var
	X,Y: Integer;
begin
	for Y:=Low(YRange) to High(YRange) do
		for X:=Low(XRange) to High(XRange) do
			with Cells[X,Y] do
			begin
				case NeighboursCount(X,Y,Cells) of
					0..1,4..8: NextState:=False;
					2: NextState:=CurState;
					3: NextState:=True;
				end;
				if (NewProbability<>0) and
					(Random(NewProbability)=0) then NextState:=True;
			end;
end;

procedure NextGeneration(var Cells: TCellsArray);
var
	X: XRange;
	Y: YRange;
begin
	for Y:=1 to High(Y) do
		for X:=1 to High(X) do Cells[X,Y].CurState:=Cells[X,Y].NextState;
end;

begin
	ClrScr;
	CursorOff;
	Randomize;
	repeat
		LastKey:=0;
		Start;
		repeat
			if KeyPressed then
			begin
				LastKey:=GetKey;
				case LastKey of
					Space:
						if OldColor=Red then OldColor:=Black
						else OldColor:=Red;
					Enter,Esc: Break;
					Left: if DelayTime>0 then Dec(DelayTime,50);
					Right: if DelayTime<500 then Inc(DelayTime,50);
					Up:
						if NewProbability=0 then NewProbability:=50000
						else
							if NewProbability>5000 then Dec(NewProbability,5000);
					Down:
						if NewProbability<>0 then
							if NewProbability=50000 then NewProbability:=0
							else
								if NewProbability<50000 then Inc(NewProbability,5000);
				end;
				LastKey:=0;
			end
			else Delay(DelayTime);
			ResetCells(Cells);
			DrawCells(Cells);
			NextGeneration(Cells);
		until False;
	until LastKey=Esc;
	ClrScr;
	CursorOn;
	NormVideo;
	{$IFNDEF PROTECTED}
	WriteConst(NewProbability,NewProbability,SizeOf(NewProbability));
	WriteConst(OldColor,OldColor,SizeOf(OldColor));
	WriteConst(DelayTime,DelayTime,sizeOf(DelayTime));
	{$ENDIF}
end.